/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import org.openide.explorer.ExplorerManager;

/**
 * Basic implementation of the Layer-interface.
 *
 * The basis for a Layer on a map.
 *
 * @author Timon Veenstra
 */
public class BaseLayer implements Layer, PropertyChangeListener {

    /**
     * Palette identifier String
     */
    /**
     * Property Identifier used when setting the ObjectLayer Interactive
     * (PropertyChange will be fired, using this id)
     */
    private Palette palette;
    private final Category category;
    private final String name;
    private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    private SelectionChangeDirection selectionChangeDirection;

    /**
     * Constructor can only be called from extending classes since this class is
     * abstract.
     *
     * @param palette Color Palette
     * @param category Category this Layer belongs to
     * @param name Name of the Layer
     */
    public BaseLayer(Palette palette, Category category, String name) {
        this.palette = palette;
        this.category = category;
        this.name = name;

        if (palette != null) {
            palette.addPropertyChangeListener(this);
        }
    }

    /**
     * @see Layer#getCategory
     * @return The category this BaseLayer belongs to
     */
    @Override
    public Category getCategory() {
        return this.category;
    }

    /**
     * @see Layer#getName Layer
     * @return The name of this BaseLayer
     */
    @Override
    public String getName() {
        return this.name;
    }
    
    /**
     * @see Layer#setPalette(Palette palette) Layer
     * @param palette The palette to set for this BaseLayer
     */
    @Override
    public final void setPalette(final Palette palette) {
        assert palette != null;
        if (this.palette != null) {
            this.palette.removePropertyChangeListener(this);
        }
        Palette old = this.palette;
        this.palette = palette;
        pcs.firePropertyChange(PROP_PALETTE, old, palette);
        palette.addPropertyChangeListener(this);
    }

    /**
     * @see Layer#getPalette Layer
     * @return The palette The palette for this BaseLayer
     */
    @Override
    public Palette getPalette() {
        return this.palette;
    }

    /**
     * Add property change listener to this class.
     * {@link nl.cloudfarming.client.model.PropertyChangeEmitter#addPropertyChangeListener(PropertyChangeListener listener)}}
     *
     * @param listener Listener to add to this BaseLayer
     */
    @Override
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        pcs.addPropertyChangeListener(listener);
    }

    /**
     * Add property change listener to this class.
     * {@link nl.cloudfarming.client.model.PropertyChangeEmitter#addPropertyChangeListener(String propertyName, PropertyChangeListener listener)}
     *
     * @param propertyName PropertyName to listen to
     * @param listener Listener to add to this BaseLayer
     */
    @Override
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        pcs.addPropertyChangeListener(propertyName, listener);
    }

    /**
     * Remove the property change listener from this class.
     * {@link nl.cloudfarming.client.model.PropertyChangeEmitter#removePropertyChangeListener(String propertyName, PropertyChangeListener listener)}
     *
     * @param propertyName PropertyName to stop listening to
     * @param listener Listener to remove
     */
    @Override
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        pcs.removePropertyChangeListener(propertyName, listener);
    }

    /**
     * Remove the property change listener from this class.
     * {@link nl.cloudfarming.client.model.PropertyChangeEmitter#removePropertyChangeListener(PropertyChangeListener listener)}
     *
     * @param listener Listener to remove
     */
    @Override
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        pcs.removePropertyChangeListener(listener);
    }

    /**
     * Returns the PropertyChangeSupport object bound to this class.
     *
     * @return The PropertyChangeSupport object bound to this class.
     */
    protected PropertyChangeSupport getChangeSupport() {
        return pcs;
    }

    /**
     * @see Layer#getActions
     * @return Array of actions for this BaseLayer
     */
    @Override
    public LayerAction[] getActions() {
        return null;
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if (Palette.PROP_BASE_COLOR.equals(evt.getPropertyName())) {
            pcs.firePropertyChange(PROP_PALETTE, null, palette);
        }
    }

    @Override
    public void fireNodeSelectionChange(PropertyChangeEvent event, SelectionChangeDirection direction) {
        assert ExplorerManager.PROP_SELECTED_NODES.equals(event.getPropertyName()):"Selection change event should always use propertyname ExplorerManager.PROP_SELECTED_NODES";
        if (selectionChangeDirection == null) {
            selectionChangeDirection = direction;
            pcs.firePropertyChange(event);
            selectionChangeDirection = null;
        }
    }

    @Override
    public SelectionChangeDirection getActiveChangeDirection() {
        return selectionChangeDirection;
    }

//    @Override
//    public boolean equals(Object obj) {
//        if (obj == null) {
//            return false;
//        }
//        if (getClass() != obj.getClass()) {
//            return false;
//        }
//        final BaseLayer other = (BaseLayer) obj;
//        if ((this.name == null) ? (other.name != null) : !this.name.equals(other.name)) {
//            return false;
//        }
//        return true;
//    }
//
//    @Override
//    public int hashCode() {
//        int hash = 5;
//        hash = 97 * hash + (this.name != null ? this.name.hashCode() : 0);
//        return hash;
//    }
    
    
}
